import(methods)
importFrom(stats, setNames)
importFrom(httr, GET, content, user_agent, stop_for_status)
importFrom(jsonlite, fromJSON)

importFrom(S4Vectors, wmsg, isTRUEorFALSE, isSingleNumber,
                      isSingleString, isSingleStringOrNA,
                      toListOfIntegerVectors)

export(
    UCSC.api.url,
    list_UCSC_genomes,
    get_organism_for_UCSC_genome,
    get_UCSC_chrom_sizes,
    list_UCSC_tracks,
    fetch_UCSC_track_data,
    UCSC_dbselect
)

